var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import prompts from 'prompts';
export function askForInputs(inputs, name, repl = null) {
    var _a, _b, _c, _d;
    return __awaiter(this, void 0, void 0, function* () {
        if (inputs) {
            if (inputs.type === 'object') {
                return yield askForObjectInputs(inputs, name, repl);
            }
            else if (inputs.type === 'string') {
                const response = yield prompts({
                    type: 'text',
                    name: 'value',
                    message: `Enter a value for ${name}.`,
                    initial: (_a = inputs.defaultValue) !== null && _a !== void 0 ? _a : undefined,
                });
                return evalResult(response.value || undefined, repl, String);
            }
            else if (inputs.type === 'number') {
                const response = yield prompts({
                    type: repl ? 'text' : 'number',
                    name: 'value',
                    message: `Enter a number for ${name}.`,
                    initial: (_b = inputs.defaultValue) !== null && _b !== void 0 ? _b : undefined,
                });
                return typeof response.value === 'number'
                    ? response.value
                    : typeof response.value === 'string'
                        ? yield evalResult(response.value || undefined, repl, Number)
                        : undefined;
            }
            else if (inputs.type === 'boolean') {
                const response = yield prompts({
                    type: repl ? 'text' : 'toggle',
                    name: 'value',
                    message: `Enable ${name}?`,
                    initial: (_c = inputs.defaultValue) !== null && _c !== void 0 ? _c : undefined,
                    active: 'yes',
                    inactive: 'no',
                });
                return typeof response.value === 'boolean'
                    ? response.value
                    : typeof response.value === 'string'
                        ? yield evalResult(response.value || undefined, repl, Boolean)
                        : undefined;
            }
            else if (inputs.type === 'date') {
                const response = yield prompts({
                    type: repl ? 'text' : 'date',
                    name: 'value',
                    message: `Enter a date for ${name}.`,
                    initial: (_d = inputs.defaultValue) !== null && _d !== void 0 ? _d : undefined,
                });
                return response.value instanceof Date
                    ? response.value
                    : typeof response.value === 'string'
                        ? yield evalResult(response.value || undefined, repl, Date)
                        : undefined;
            }
            else if (inputs.type === 'literal') {
                return inputs.value;
            }
            else if (inputs.type === 'null') {
                return null;
            }
            else if (inputs.type === 'array') {
                return yield askForArrayInputs(inputs, name, repl);
            }
            else if (inputs.type === 'enum') {
                return yield askForEnumInputs(inputs, name, repl);
            }
            else if (inputs.type === 'union') {
                return yield askForUnionInputs(inputs, name, repl);
            }
            else if (inputs.type === 'any') {
                return yield askForAnyInputs(name, repl);
            }
        }
        return undefined;
    });
}
function evalResult(result, repl, type) {
    return __awaiter(this, void 0, void 0, function* () {
        if (typeof result === 'string' && repl && result.startsWith('.')) {
            const script = result.slice(1);
            if (script.startsWith('.')) {
                // Scripts can be escaped with a double dot.
                return script;
            }
            const promise = new Promise((resolve, reject) => {
                repl.eval(result.slice(1), repl.context, 'repl', (err, value) => {
                    if (err) {
                        reject(err);
                        return;
                    }
                    resolve(value);
                });
            });
            const value = yield promise;
            return value;
        }
        if (type && result !== undefined && result !== null && result !== '') {
            return type(result);
        }
        return result;
    });
}
function askForObjectInputs(inputs, name, repl) {
    return __awaiter(this, void 0, void 0, function* () {
        const result = {};
        for (let key in inputs.schema) {
            const prop = inputs.schema[key];
            const value = yield askForInputs(prop, `${name}.${key}`, repl);
            result[key] = value;
        }
        return result;
    });
}
function askForArrayInputs(inputs, name, repl) {
    var _a, _b;
    return __awaiter(this, void 0, void 0, function* () {
        const result = [];
        if (inputs.optional || inputs.nullable) {
            const response = yield prompts({
                type: 'confirm',
                name: 'continue',
                message: `Do you want to enter an array for ${name}?`,
            });
            if (!response.continue) {
                if (inputs.nullable) {
                    return null;
                }
                else {
                    return undefined;
                }
            }
        }
        let length = 0;
        if (typeof inputs.exactLength === 'number') {
            length = inputs.exactLength;
        }
        else {
            const response = yield prompts({
                type: 'number',
                name: 'length',
                message: `Enter the length of the array for ${name}.`,
                min: (_a = inputs.minLength) !== null && _a !== void 0 ? _a : undefined,
                max: (_b = inputs.maxLength) !== null && _b !== void 0 ? _b : undefined,
            });
            length = response.length;
        }
        for (let i = 0; i < length; i++) {
            const value = yield askForInputs(inputs.schema, `${name}[${i}]`, repl);
            result.push(value);
        }
        return result;
    });
}
function askForEnumInputs(inputs, name, repl) {
    return __awaiter(this, void 0, void 0, function* () {
        let choices = inputs.values.map((value) => ({
            title: value,
            value: value,
        }));
        if (inputs.nullable) {
            choices = [
                {
                    title: '(null)',
                    value: '',
                },
                ...choices,
            ];
        }
        if (inputs.optional) {
            choices = [
                {
                    title: '(undefined)',
                    value: '',
                },
                ...choices,
            ];
        }
        const response = yield prompts({
            type: 'select',
            name: 'value',
            message: `Select a value for ${name}.`,
            choices: choices,
        });
        if (response.value === '') {
            if (inputs.nullable) {
                return null;
            }
            else {
                return undefined;
            }
        }
        return response.value;
    });
}
function askForUnionInputs(inputs, name, repl) {
    return __awaiter(this, void 0, void 0, function* () {
        if ('discriminator' in inputs) {
            return yield askForDiscriminatedUnionInputs(inputs, name, repl);
        }
        else {
            const kind = yield prompts({
                type: 'select',
                name: 'kind',
                message: `Select a kind for ${name}.`,
                choices: inputs.options.map((option) => {
                    if (option.type === 'literal') {
                        return {
                            title: `(${option.value})`,
                            description: option.description,
                            value: option.value,
                        };
                    }
                    else if (option.type === 'null') {
                        return {
                            title: `(null)`,
                            description: option.description,
                            value: null,
                        };
                    }
                    return {
                        title: option.type,
                        description: option.description,
                        value: option,
                    };
                }),
            });
            return yield askForInputs(kind.kind, name, repl);
        }
    });
}
function askForDiscriminatedUnionInputs(inputs, name, repl) {
    return __awaiter(this, void 0, void 0, function* () {
        const kind = yield prompts({
            type: 'select',
            name: 'kind',
            message: `Select a ${inputs.discriminator} for ${name}.`,
            choices: inputs.options.map((option) => {
                const prop = option.schema[inputs.discriminator];
                if (prop.type !== 'literal') {
                    return {
                        title: option.type,
                        description: option.description,
                        value: option,
                    };
                }
                return {
                    title: prop.value,
                    description: option.description,
                    value: option,
                };
            }),
        });
        return yield askForInputs(kind.kind, name, repl);
    });
}
function askForAnyInputs(name, repl) {
    return __awaiter(this, void 0, void 0, function* () {
        const kind = yield prompts({
            type: 'select',
            name: 'kind',
            message: `Select a kind for ${name}.`,
            choices: [
                {
                    title: 'json',
                    description: 'A JSON value.',
                    value: 'json',
                },
                {
                    title: 'string',
                    description: 'A string value.',
                    value: 'string',
                },
                {
                    title: 'number',
                    description: 'A number value.',
                    value: 'number',
                },
                {
                    title: 'boolean',
                    description: 'A boolean value.',
                    value: 'boolean',
                },
                {
                    title: '(null)',
                    description: 'A null value.',
                    value: 'null',
                },
            ],
        });
        if (kind.kind === 'null') {
            return null;
        }
        else if (kind.kind === 'json') {
            while (true) {
                try {
                    const response = yield prompts({
                        type: 'text',
                        name: 'value',
                        message: `Enter a JSON value for ${name}.`,
                    });
                    if (response.value === '' || response.value === undefined) {
                        return undefined;
                    }
                    const script = response.value;
                    if (typeof script === 'string' && script.startsWith('.')) {
                        return yield evalResult(script, repl, null);
                    }
                    return JSON.parse(response.value);
                }
                catch (err) {
                    console.log('Invalid JSON value.', err);
                }
            }
        }
        return yield askForInputs({
            type: kind.kind,
            nullable: true,
            optional: true,
        }, name, repl);
    });
}
//# sourceMappingURL=schema.js.map